==> jungle-1.0.tar.gz: jungle-1.0/jungle/apps/timereport/profile_form.py <==
from django.newforms.models import form_for_model
from django.newforms import fields
from jungle.templatetags.jungle_tags import ugettext as _
from models import TimetableProfile, TimetableProfileDetail, time2sec
from jungle.ref.models import WeekDay 
from django.newforms.util import ErrorList, ValidationError, smart_unicode
from jungle.forms import fields as jfields
from datetime import *
import re
from django.newforms.models import form_for_model, form_for_instance

COLS = (1, 2, 3, 4) ## how many columns to represent timetable

class ProfileForm(form_for_model(TimetableProfile)):

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)

        if 'time_1_1' not in self.data:
            self.data.update(get_day_details(self.data.get('id')))

        opts = {'required' : False,}
        #'initial' : '00:00' }
        for d in range(1,8):
            for j in COLS:
                self.fields['time_%s_%s' % (d, j)] = jfields.JungleTimeField(**opts) 

    def save(self):
        if 'id' in self.data:
            ## update
            self.profile = TimetableProfile.objects.get(id=self.data.get('id'))
            f = form_for_instance(self.profile)(self.data)
            f.save()
            #self.profile.save()
        else: ##new
            self.profile = super(ProfileForm, self).save()

        self.save_detail()

        return self.profile

    def save_detail(self):
        """A normal day (8:00-12:00 / 14:00-18:00) would be split
        into 2 entries with ascending odv 1,2,3,4
        """
        for d in range(1,8):
            day = WeekDay.objects.get(isoweekday=d)

            for col in COLS:
                if col % 2 == 0:  # only official inbound transits (odd numbers)
                    continue
                try:
                    ## update
                    t = TimetableProfileDetail.objects.get(
                        profile=self.profile,
                        isoweekday=day,
                        odv=col,
                        )
                    if self.cleaned_data['time_%s_%s' % (d, col)]:
                        t.time = self.cleaned_data['time_%s_%s' % (d, col)]
                        t.time_out = self.cleaned_data['time_%s_%s' % (d, col+1)]
                        t.save()
                    else:
                        t.delete()
                except:
                    # new record
                    if self.cleaned_data['time_%s_%s' % (d, col)]:
                        t = TimetableProfileDetail(
                            profile=self.profile,
                            isoweekday=day,
                            odv=col,
                            time=self.cleaned_data['time_%s_%s' % (d, col)],
                            time_out=self.cleaned_data['time_%s_%s' % (d, col+1)],
                            #direction=(col+1)%2+1  # 1 in, 2 out (as in transit)
                            )
                        
                        t.save()


def get_day_details(object_id):
    """get all days from the profileDetail table, collect them in a single var
       to be displayed in the template
    """
    from jungle.ref.models import WeekDay
    from jungle.functions.utils import DictWithDefault
    days = DictWithDefault(default='')
    
    TT = TimetableProfileDetail.objects.filter(profile=object_id).order_by('id')
    for T in TT:
        days['time_%s_%s' % (T.isoweekday.isoweekday, T.odv)] = T.time
        days['time_%s_%s' % (T.isoweekday.isoweekday, T.odv+1)] = T.time_out

    return days

==> jungle-1.0.2.tar.gz: jungle-1.0.2/jungle/apps/timereport/profile_form.py <==
from django.newforms.models import form_for_model
from django.newforms import fields
from jungle.templatetags.jungle_tags import ugettext as _
from models import TimetableProfile, TimetableProfileDetail, time2sec
from jungle.ref.models import WeekDay
from django.newforms.util import ErrorList, ValidationError, smart_unicode
from jungle.forms import fields as jfields
from datetime import *
import re
from django.newforms.models import form_for_model, form_for_instance

COLS = (1, 2, 3, 4)  # how many columns to represent timetable

class ProfileForm(form_for_model(TimetableProfile)):

    def __init__(self, *args, **kwargs):
        super(ProfileForm, self).__init__(*args, **kwargs)

        if 'time_1_1' not in self.data:
            self.data.update(get_day_details(self.data.get('id')))

        opts = {'required': False, }
        #'initial' : '00:00' }
        for d in range(1, 8):
            for j in COLS:
                self.fields['time_%s_%s' % (d, j)] = jfields.JungleTimeField(**opts)

    def save(self):
        if 'id' in self.data:
            ## update
            self.profile = TimetableProfile.objects.get(id=self.data.get('id'))
            f = form_for_instance(self.profile)(self.data)
            f.save()
            #self.profile.save()
        else:  # new
            self.profile = super(ProfileForm, self).save()

        self.save_detail()

        return self.profile

    def save_detail(self):
        """A normal day (8:00-12:00 / 14:00-18:00) would be split
        into 2 entries with ascending odv 1,2,3,4
        """
        for d in range(1, 8):
            day = WeekDay.objects.get(isoweekday=d)

            for col in COLS:
                if col % 2 == 0:  # only official inbound transits (odd numbers)
                    continue
                try:
                    ## update
                    t = TimetableProfileDetail.objects.get(
                        profile=self.profile,
                        isoweekday=day,
                        odv=col,
                    )
                    if self.cleaned_data['time_%s_%s' % (d, col)]:
                        t.time = self.cleaned_data['time_%s_%s' % (d, col)]
                        t.time_out = self.cleaned_data['time_%s_%s' % (d, col+1)]
                        t.save()
                    else:
                        t.delete()
                except:
                    # new record
                    if self.cleaned_data['time_%s_%s' % (d, col)]:
                        t = TimetableProfileDetail(
                            profile=self.profile,
                            isoweekday=day,
                            odv=col,
                            time=self.cleaned_data['time_%s_%s' % (d, col)],
                            time_out=self.cleaned_data['time_%s_%s' % (d, col+1)],
                            #direction=(col+1)%2+1  # 1 in, 2 out (as in transit)
                        )

                        t.save()


def get_day_details(object_id):
    """get all days from the profileDetail table, collect them in a single var
       to be displayed in the template
    """
    from jungle.ref.models import WeekDay
    from jungle.functions.utils import DictWithDefault
    days = DictWithDefault(default='')

    TT = TimetableProfileDetail.objects.filter(profile=object_id).order_by('id')
    for T in TT:
        days['time_%s_%s' % (T.isoweekday.isoweekday, T.odv)] = T.time
        days['time_%s_%s' % (T.isoweekday.isoweekday, T.odv+1)] = T.time_out

    return days
